# from .SharePointAPI import SharePointAPI as SP
from sharepoint_api.SharePointTimeRegistration import TimeRegistration
from .SharePointListItem import SharePointListItem, SharepointSiteCase
from typing import List
import json

class SharePointList:
    '''
    '''

    settings = {}
    _items = None
    sharepoint_site = ""
    SPItem = SharePointListItem

    CHANGE_DETECTED = False
    SAVE_ON_CHANGE = False
    JSON_FILENAME = None

    def __init__(self, sharepoint_site, settings: dict = None, items: List[SPItem] = None):
        self.sharepoint_site = sharepoint_site
        self.settings = settings
        self._items = items

    def __str__(self):
        items = ''
        for _item in self.all_items:
            items = items+str(_item.Title)+'\n'
        return items

    def __del__(self):
        if self.CHANGE_DETECTED and self.SAVE_ON_CHANGE and self.JSON_FILENAME is not None:
            print('Change was definiteley detected')
            print('Saving items')
            self.save_as_json(self.JSON_FILENAME)

    @property
    def all_items(self) -> List[SPItem]:
        '''
            Get list of all SharePointListItem objects
        '''
        if self._items is None:
            self._items = []
        return self._items

    @property
    def Title(self) -> str:
        return self.settings['Title']

    @property
    def guid(self):
        return self.settings['Id']

    def append_items(self, items):

        if isinstance(items, list):
            for item in items:
                item._list = self
            _items = self.all_items.extend(items)
            self.CHANGE_DETECTED = True
        elif isinstance(items, self.SPItem):
            items._list = self
            _items = self.all_items.append(items)
            self.CHANGE_DETECTED = True

    def get_item_by_name(self, name):
        '''
        '''
        for item in self.all_items:
            if name ==item.Title:
                return item
        return None

    def get_item_by_id(self, id):
        '''
        '''
        for item in self._items:
            if id ==item.Id:
                return item
        return None

    def get_items_by_assigned_id(self, id) -> List[SPItem]:
        '''
        '''
        items = []
        for item in self._items:
            if id == item.ResponsibleId:
                items.append(item)
        return items

    def save_as_json(self, file_name):

        out_dict = {
            'sharepoint_site': self.sharepoint_site,
            'GUID': self.guid,
            'Settings': self.settings,
            "cases": [{'settings': case.settings, 'versions': case._versions} for case in self.all_items]
            }

        with open(file_name, 'w') as fp:
            json.dump(out_dict, fp)
        
        self.CHANGE_DETECTED = False


class CasesList(SharePointList):
    SPItem = SharepointSiteCase

class TimeRegistrationList(SharePointList):
    SPItem = TimeRegistration



from datetime import datetime

from sharepoint_api.SharePointListItem import SharePointListItem
# from .SharePointAPI import SharePointAPI as SP

class TimeRegistration(SharePointListItem):
    ''' 
    '''

    _DoneBy = None

    @property
    def Created(self) -> datetime:
        'Timestamp of when the item was created'
        if 'DoneDate' not in self.settings:
            return None
        elif self.settings['DoneDate'] is None:
            return None
        else:
            return datetime.strptime(self.settings['DoneDate'], '%Y-%m-%dT%H:%M:%SZ')
    
    @property
    def Hours(self) -> str:
        return self.settings['Hours']

    @property
    def DoneById(self) -> str:
        return self.settings['DoneById']

    @property
    def DoneBy(self) -> str:
        if not self._DoneBy:
            user_list = self.sp.get_users(self.sharepoint_site)
            self._DoneBy = user_list.get_user_by_id(self.DoneById)
        return self._DoneBy
    
    @property
    def DoneUsername(self) -> str:
        return self.DoneBy.UserName

    @property
    def CaseId(self) -> str:
        return self.settings['CaseId']
    
    @property
    def WorkPackageId(self) -> str:
        return self.settings['WorkPackageId']
    



class SharePointUser:

    settings = None
    _users = None
    sharepoint_site = None

    def __init__(self, settings: dict = None):
        self.settings = settings

    @property
    def Id(self):
        if self.settings is None:
            return None
        return self.settings['Id']

    @property
    def Title(self):
        if self.settings is None:
            return None
        return self.settings['Title']

    @property
    def UserName(self):
        if self.settings is None:
            return None
        return self.settings['Email'].split('@')[0].lower()

    @property
    def Email(self):
        if self.settings is None:
            return None
        return self.settings['Email']    




class SharePointUserList:

    _users = []
    sharepoint_site = ''

    def __init__(self, sharepoint_site, users):
        self._users = users
        self.sharepoint_site = sharepoint_site

    def __str__(self):
        n_users = len(self._users)
        if n_users == 1:
            return 'A list of 1 user'

        return f'A list of {n_users} users'
    
    def get_user_by_username(self, name):
        for user in self._users:
            if name == user.UserName:
                return user
        return

    def get_user_by_id(self, user_id):
        for user in self._users:
            if user_id == user.Id:
                return user
        return




from .SharePointList import SharePointList, CasesList
from typing import List

class SharePointLists:

    _lists = None

    def __init__(self, lists):
        self._lists = lists

    def __str__(self):
        lists = ''
        for _list in self.all_lists:
            lists = lists+str(_list.Title)+'\n'
        return lists

    @property
    def all_lists(self) -> List[SharePointList]:
        '''
            Get list of all SharePointList objects
        '''
        if self._lists is None:
            self._lists = []
        return self._lists

    @property
    def all_list_titles(self) -> str:
        '''
            Get list of all SharePointList objects
        '''
        
        return [_list.Title for _list in self.all_lists]


    def get_list(self, key, SPListType: SharePointList=SharePointList) -> SharePointList:
        '''
            Get SharePointList by:
                - Index: int
                - List title: str
        '''
        if isinstance(key, int):
            return self.all_lists[key]
        elif isinstance(key, str):
            for list_object in self.all_lists:
                if list_object.Title == key:
                    return list_object
            raise KeyError("No list with title '{0}'".format(key))
        raise KeyError

    def get_cases_list(self, key) -> CasesList:
        '''
            Get SharePointList by:
                - Index: int
                - List title: str
        '''

        list_object = self.get_list(key, SPListType=CasesList)

        return CasesList(list_object.sharepoint_site, list_object.settings)



from datetime import datetime
# from .SharePointAPI import SharePointAPI as SP

class SharePointListItem:
    ''' 
    '''
    settings = {}

    def __init__(self, sp, sharepoint_site, list_guid, settings: dict = None, versions: list = None):
        self.sp = sp
        self.sharepoint_site = sharepoint_site
        self.list_guid = list_guid
        self._list = None
        self._versions = versions
        self.settings = settings

    def __str__(self):
        return self.Title

    @property
    def list(self):
        if not self._list:
            self._list = self.sp.get_list(self.sharepoint_site, self.list_guid)
        return self._list

    @property
    def versions(self) -> list:
        if not self._versions:
            self._versions = self.sp.get_item_versions(self.sharepoint_site, self.list_guid, self.Id) 
            if self._list is not None:
                self._list.CHANGE_DETECTED = True
        return self._versions
    
    
    @property
    def Id(self) -> str:
        return self.settings['Id']

    @property
    def Title(self) -> str:
        return self.settings['Title']

    @property
    def Created(self) -> datetime:
        'Timestamp of when the item was created'
        if 'Created' not in self.settings:
            return None
        elif self.settings['Created'] is None:
            return None
        else:
            return datetime.strptime(self.settings['Created'], '%Y-%m-%dT%H:%M:%SZ')

    @property
    def Modified(self) -> datetime:
        'Timestamp of when the item was modified'
        if 'Modified' not in self.settings:
            return None
        elif self.settings['Modified'] is None:
            return None
        else:
            return datetime.strptime(self.settings['Modified'], '%Y-%m-%dT%H:%M:%SZ')

    def attach_item(self, file_name, file_path):
        with open(file_path, 'r') as f:
            file_content = f.read()

    
        self.sp.attach_file(self.sharepoint_site, self.list, self, file_name, file_content)
    


class SharepointSiteCase(SharePointListItem):
    
    @property
    def AssignmentType(self) -> str:
        return self.settings['AssignmentType']

    @property
    def CaseClosedTimestamp(self) -> datetime:
        'Timestamp of when the case was closed'
        if 'CaseClosed' not in self.settings:
            return None
        elif self.settings['CaseClosed'] is None:
            return None
        else:
            return datetime.strptime(self.settings['CaseClosed'], '%Y-%m-%dT%H:%M:%SZ')

    @property
    def Due(self) -> datetime:
        'Timestamp of when the case is due'
        if 'DueDate' not in self.settings:
            return None
        elif self.settings['DueDate'] is None:
            return None
        else:
            return datetime.strptime(self.settings['DueDate'], '%Y-%m-%dT%H:%M:%SZ')

    @property
    def Priority(self) -> str:
        return self.settings['Priority']

    @property
    def ResponsibleId(self) -> str:
        return self.settings['AssignedToId']

    @property
    def Status(self) -> str:
        return self.settings['Status']

    @property
    def SolvedInTime(self) -> bool:
        'Bool indicating whether work was solved in time'
        solved_in_time = False
        if self.Due:
            if not self.CaseClosedTimestamp:
                solved_in_time = True
            elif self.CaseClosedTimestamp <= self.Due:
                solved_in_time = True
        else:
            solved_in_time = True
        
        return solved_in_time

    @property
    def WorkBegunTimestamp(self) -> datetime:
        'Timestamp of when work was started on the case'
        if 'WorkBegun' not in self.settings:
            return None
        elif self.settings['WorkBegun'] is None:
            return None
        else:
            return datetime.strptime(self.settings['WorkBegun'], '%Y-%m-%dT%H:%M:%SZ')

    @property
    def DeadlineWorkTimestamp(self) -> datetime:
        'Timestamp of when work should start on the case'
        if 'DeadlineWork' not in self.settings:
            return None
        elif self.settings['DeadlineWork'] is None:
            return None
        else:
            return datetime.strptime(self.settings['DeadlineWork'], '%Y-%m-%dT%H:%M:%SZ')

    @property
    def ReactedInTime(self) -> bool:
        'Bool indicating whether work was started in time'
        reacted_in_time = False
        if self.DeadlineWorkTimestamp:
            if not self.WorkBegunTimestamp:
                reacted_in_time = True
            elif self.WorkBegunTimestamp <= self.DeadlineWorkTimestamp:
                reacted_in_time = True
        else:
            reacted_in_time = True
        return reacted_in_time

    
        



import sys
import os
import getpass
import json
import requests
from requests_ntlm import HttpNtlmAuth
from typing import List

from .SharePointUser import SharePointUser
from .SharePointUserList import SharePointUserList
from .SharePointListItem import SharePointListItem, SharepointSiteCase
from .SharePointList import SharePointList, CasesList, TimeRegistrationList
from .SharePointLists import SharePointLists


class SharePointAPI:

    @classmethod
    def _compact_init(cls, credentials: dict):
        '''
        Takes credentials in the form of a dict
        {
            "username": "",
            "password": "",
            "sharepoint_url": ""
        }
        '''

        username = credentials['username']
        password = credentials['password']
        sharepoint_url = credentials['sharepoint_url']
        proxies = credentials['proxies']


        return cls(username, password, sharepoint_url, proxies)

    def __init__(self, username: str, password: str, sharepoint_url: str, proxies: dict):
        '''
        
        '''

        self.username = username
        self.password = password
        self.sharepoint_url = sharepoint_url
        self.proxies = proxies

    def _api_post_call(self, url, post_data, form_digest_value=None, merge=False):
        headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:71.0) Gecko/20100101 Firefox/71.0',
            'accept': 'application/json;odata=verbose',
            'Content-type': 'application/json; charset=utf-8',
            'Cache-Control': 'no-cache',
            'Connection': 'keep-alive',
            'Accept-Encoding': 'gzip, deflate, br',
            'Content-Length': str(len(f'{post_data}')),
            'X-RequestDigest': f'{form_digest_value}',
            'If-Match': '*'
        }
        if merge:
            headers['X-HTTP-Method'] = 'MERGE'



        r = requests.post(url,
                         auth=HttpNtlmAuth(self.username, self.password), headers=headers, json=post_data, proxies=self.proxies)

        if r.status_code == 400:
            print('Failed to connect: ', r.status_code)
            print(r.text)
            print('Exiting.')
            raise ConnectionError
        elif r.status_code == 404:
            print('Failed to connect: ', r.status_code)
            print(r.request.url)
            print(r.request.body)
            print('Exiting.')
            raise ConnectionError
        elif r.status_code not in [200, 201, 204]:
            print('Failed to connect: ', r.status_code)
            print('Exiting.')
            raise ConnectionError
        return r

    def _api_attachment_call(self, url, post_data, form_digest_value=None, overwrite=False, x_http_method=None):
        headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:71.0) Gecko/20100101 Firefox/71.0',
            'accept': 'application/json;odata=verbose',
            'X-RequestDigest': f'{form_digest_value}'
        }
        if overwrite:
            headers['X-HTTP-Method'] = "PUT"
        if x_http_method:
            if x_http_method.lower() == 'delete':
                headers['X-HTTP-Method'] = "DELETE"
            elif x_http_method.lower() == 'put':
                headers['X-HTTP-Method'] = "PUT"
            else:
                print(f'X-HTTP-Method "{x_http_method}" is not implemented')

        if post_data:
            headers['Content-Length'] = str(len(f'{post_data}'))
            r = requests.post(url,
                         auth=HttpNtlmAuth(self.username, self.password), data=post_data, headers=headers, proxies=self.proxies)
        else:
            r = requests.post(url,
                         auth=HttpNtlmAuth(self.username, self.password), headers=headers, proxies=self.proxies)


        if r.status_code == 404:
            print('Failed to connect: ', r.status_code)
            print(r.request.url)
            # print(r.request.body)
            print(str(r.content))
            print('Exiting.')
            raise ConnectionError
        if r.status_code == 400:
            print('Failed to connect: ', r.status_code)
            print(r.reason)
            # print(r.request.body)
            print(str(r.content))
            print('Exiting.')
            raise ConnectionError
        elif r.status_code not in [200, 204]:
            print('Failed to connect: ', r.status_code)
            try:
                print(r.json()['error']['message']['value'])
            except:
                print('No error message found')
            print('Exiting.')
            raise ConnectionError
        return r

    def _api_get_call(self, url, *args, **kwargs):
        headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:71.0) Gecko/20100101 Firefox/71.0',
            'accept': 'application/json;odata=verbose',
            'Content-type': 'application/json; charset=utf-8',
            'Cache-Control': 'no-cache',
            'Connection': 'keep-alive',
            'Accept-Encoding': 'gzip, deflate, br'
        }

        r = requests.get(url,
                         auth=HttpNtlmAuth(self.username, self.password), headers=headers, proxies=self.proxies)


        if r.status_code == 404:
            print('Failed to process request with status code ', r.status_code)
            print(r.json()['error']['message']['value'])
            raise ConnectionError
        elif r.status_code != 200:
            print('Failed to connect: ', r.status_code)
            try:
                print(json.loads(r.text)['error']['message']['value'])
            except:
                pass
            try:
                print(json.loads(r.text)['error']['code'])
            except:
                pass
            json.loads(r.text)
            print('Exiting.')
            raise ConnectionError
        return r

    def get_users(self, sharepoint_site):
        '''
            Returns a list of users from a given sharepoint_site
        '''
        r = self._api_get_call(f'{self.sharepoint_url}/cases/{sharepoint_site}/_api/Web/siteusers')

        users = []

        for user_settings in r.json()["d"]["results"]:
            users.append(SharePointUser(user_settings))

        return SharePointUserList(sharepoint_site, users)

    def get_user(self, sharepoint_site, user_id):
        '''
            Returns a list of users from a given sharepoint_site
        '''
        if user_id is None:
            return SharePointUser()
        r = self._api_get_call(f'{self.sharepoint_url}/cases/{sharepoint_site}/_api/Web/siteusers?$filter=( Id eq {user_id} )')

        user_settings = r.json()["d"]["results"][0]
        return SharePointUser(user_settings)

    # SP Lists

    def get_lists(self, sharepoint_site):
        '''
            Returns a list of lists from a given sharepoint_site
        '''
        r = self._api_get_call(f'{self.sharepoint_url}/cases/{sharepoint_site}/_api/Web/Lists')

        _lists = []
        for list_props in r.json()["d"]["results"]:
                _lists.append(SharePointList(sharepoint_site, list_props))

        return SharePointLists(_lists)

    def get_list(self, sharepoint_site, sp_list, filters=None, top=1000, view_path=None, SPListType: SharePointList=SharePointList) -> SharePointList:
        '''
            Returns a list from a given sharepoint_site using its guid

            Returns a subset of items from a list

            sharepoint_site:
            guid: the guid of the list to retrieve items from
        '''

        # Uses either guid or SharePointList
        if isinstance(sp_list, SharePointList):
            guid = sp_list.guid
            r = self._api_get_call(f'{self.sharepoint_url}/cases/{sharepoint_site}/_api/Web/Lists(guid\'{guid}\')')
            sp_list = SPListType(sharepoint_site, r.json()["d"])

        elif isinstance(sp_list, str):
            guid = sp_list
            r = self._api_get_call(f'{self.sharepoint_url}/cases/{sharepoint_site}/_api/Web/Lists(guid\'{guid}\')')
            sp_list = SPListType(sharepoint_site, r.json()["d"])
        else:
            sys.exit(1)

        arguments = []
        
        if filters is not None:
            if not isinstance(filters, list):
                filter_string = self.py2sp_conditional(filters)
                # print(filters)
                # raise('invalid search filters')
            else:
            
                filter_string = self.py2sp_conditional(' and '.join(filters))
            arguments.append(f'$filter={filter_string}')
            
        if view_path is not None:
            # Shows top x items
            arguments.append(f'$ViewPath={view_path}')
        
        if top is not None:
            # Shows top x items
            arguments.append(f'$top={top}')

        r = self._api_get_call(f'{self.sharepoint_url}/cases/{sharepoint_site}/_api/Web/Lists(guid\'{guid}\')/items?{"&".join(arguments)}')

        items = [SPListType.SPItem(self, sharepoint_site, guid, item_settings) for item_settings in r.json()["d"]["results"]]
        sp_list.append_items(items)
        return sp_list

    def get_list_by_name(self, sharepoint_site, sp_list_name: str, filters=None, top=1000, view_path=None, SPListType: SharePointList=SharePointList) -> SharePointList:
        '''
            Returns a list from a given sharepoint_site filtering by list name

            Returns a subset of items from a list

            sharepoint_site: The sharepoint_site containing the list
            sp_list_name: the name of the list
            filters: query filters
            top: Maximum items to query from the list
        '''

        sp_lists = self.get_lists(sharepoint_site)

        if sp_list_name in sp_lists.all_list_titles:
            return self.get_list(sharepoint_site, sp_lists.get_list(sp_list_name), filters, top, view_path, SPListType=SPListType)
        else:
            print(f"List '{sp_list_name}' does not exist in sharepoint_site {sharepoint_site}'")
            sys.exit(1)

    def get_list_from_json(self, file_name, SPListType: SharePointList=SharePointList) -> SharePointList:
        '''
            Returns a list from a sharepoint_site based on a json file.

            file_name: the json file to load the list from

        '''
        try:
            with open(file_name, 'r') as fp:
                data_dict = json.load(fp)

            sharepoint_site = data_dict['sharepoint_site']
            guid = data_dict['GUID']

            cases = []
            for case in data_dict['cases']:
                settings = case["settings"]
                versions = None if "versions" not in case else case["versions"]
                cases.append(SPListType.SPItem(self, sharepoint_site, guid, settings, versions))

            return SPListType(sharepoint_site, data_dict['Settings'], cases)
        except FileNotFoundError as err:
            print(f"File '{file_name}' was not found")
            raise err
        except Exception as err:
            raise err

    # SP Item

    def get_item(self, sharepoint_site, sp_list, item_id) -> SharePointListItem:
        '''
            Returns a list of users from a given sharepoint_site
        '''

        if isinstance(sp_list, SharePointList):
            guid = sp_list.guid

        elif isinstance(sp_list, str):
            guid = sp_list
        else:
            sys.exit(1)

        r = self._api_get_call(f'{self.sharepoint_url}/cases/{sharepoint_site}/_api/Web/Lists(guid\'{guid}\')/items({item_id})')

        settings = r.json()["d"]
        return SharePointListItem(self, sharepoint_site, guid, settings)

    def create_item(self, sharepoint_site, sp_list, data) -> SharePointListItem:
        # Uses either guid or SharePointList
        if isinstance(sp_list, SharePointList):
            guid = sp_list.guid
        elif isinstance(sp_list, str):
            guid = sp_list
            r = self._api_get_call(f'{self.sharepoint_url}/cases/{sharepoint_site}/_api/Web/Lists(guid\'{guid}\')')
            sp_list = SharePointList(sharepoint_site, r.json()["d"]["results"][0])


        r = self._api_post_call(f'{self.sharepoint_url}/cases/{sharepoint_site}/_api/contextinfo', {})

        form_digest_value = r.json()["d"]["GetContextWebInformation"]["FormDigestValue"]

        r = self._api_post_call(f'{self.sharepoint_url}/cases/{sharepoint_site}/_api/Web/Lists(guid\'{guid}\')/items', data, form_digest_value)

        settings = r.json()["d"]
        return SharePointListItem(self, sharepoint_site, guid, settings)

    def update_item(self, sharepoint_site, sp_list, item_id, data) -> None:
        '''
            Update a sharepoint item

            sharepoint_site: The sharepoint_site containing the item
            sp_list: The list containing the item
            item_id: The id of the item
            data: Data to push to the item
        '''
        if isinstance(sp_list, SharePointList):
            guid = sp_list.guid
        elif isinstance(sp_list, str):
            guid = sp_list
            r = self._api_get_call(f'{self.sharepoint_url}/cases/{sharepoint_site}/_api/Web/Lists(guid\'{guid}\')')
            sp_list = SharePointList(sharepoint_site, r.json()["d"]["results"][0])
        else:
            raise TypeError('Only "SharePointList" and "str" types are allowed')


        r = self._api_post_call(f'{self.sharepoint_url}/cases/{sharepoint_site}/_api/contextinfo', {})

        form_digest_value = r.json()["d"]["GetContextWebInformation"]["FormDigestValue"]

        r = self._api_post_call(f'{self.sharepoint_url}/cases/{sharepoint_site}/_api/Web/Lists(guid\'{guid}\')/items({item_id})', data, form_digest_value, merge=True)

    def attach_file(self, sharepoint_site, sp_list, item, file_name, file_content) -> dict:
        # Uses either guid or SharePointList
        if isinstance(sp_list, SharePointList):
            guid = sp_list.guid
        elif isinstance(sp_list, str):
            guid = sp_list
        else:
            raise TypeError('Only "SharePointList" and "str" types are allowed')

        r = self._api_post_call(f'{self.sharepoint_url}/cases/{sharepoint_site}/_api/contextinfo', {})
        form_digest_value = r.json()["d"]["GetContextWebInformation"]["FormDigestValue"]

        r = self._api_attachment_call(f"{self.sharepoint_url}/cases/{sharepoint_site}/_api/Web/Lists(guid\'{guid}\')/items({item.Id})/AttachmentFiles/ add(FileName='{file_name}')", file_content, form_digest_value)

        return r.json()

    def get_item_versions(self, sharepoint_site, sp_list, item_id) -> list:
        '''
            Returns a list of users from a given sharepoint_site
        '''

        if isinstance(sp_list, SharePointList):
            guid = sp_list.guid

        elif isinstance(sp_list, str):
            guid = sp_list
        else:
            sys.exit(1)

        r = self._api_get_call(f'{self.sharepoint_url}/cases/{sharepoint_site}/_api/Web/Lists(guid\'{guid}\')/items({item_id})/Versions')

        versions = r.json()["d"]['results']
        return versions

    # Cases

    def get_cases_list_from_json(self, file_name) -> CasesList:
        '''
            Returns a cases list from a sharepoint_site based on a json file.

            file_name: the json file to load the list from

        '''

        return self.get_list_from_json(file_name, SPListType=CasesList)

    def get_cases_list(self, sharepoint_site, sp_list, filters=None, top=1000, view_path=None) -> CasesList:
        '''
            Returns a cases list from a given sharepoint_site using its guid

            Returns a subset of items from a list

            sharepoint_site: The sharepoint_site containing the list
            sp_list: the guid of the list to retrieve items from
            filters: query filters
            top: Maximum items to query from the list
        '''

        return self.get_list(sharepoint_site, sp_list, filters, top, view_path, SPListType=CasesList)

    def get_cases_list_by_name(self, sharepoint_site, sp_list_name: str, filters=None, top=1000, view_path=None) -> CasesList:
        '''
            Returns a cases list from a given sharepoint_site filtering by list name

            sharepoint_site: The sharepoint_site containing the list
            sp_list_name: the name of the list
            filters: query filters
            top: Maximum items to query from the list
        '''
        return self.get_list_by_name(sharepoint_site, sp_list_name, filters, top, view_path, SPListType=CasesList)
    
    def get_case(self, sharepoint_site, sp_list, item_id) -> SharePointListItem:
        '''
            Returns a list of users from a given sharepoint_site
        '''

        if isinstance(sp_list, SharePointList):
            guid = sp_list.guid

        elif isinstance(sp_list, str):
            guid = sp_list
        else:
            sys.exit(1)

        r = self._api_get_call(f'{self.sharepoint_url}/cases/{sharepoint_site}/_api/Web/Lists(guid\'{guid}\')/items({item_id})')

        settings = r.json()["d"]
        return SharepointSiteCase(self, sharepoint_site, guid, settings)

    # Time Registration

    def get_time_registration_list_from_json(self, file_name) -> TimeRegistrationList:
        '''
            Returns a Time Registration list from a sharepoint_site based on a json file.

            file_name: the json file to load the list from

        '''

        return self.get_list_from_json(file_name, SPListType=TimeRegistrationList)

    def get_time_registration_list(self, sharepoint_site, sp_list, filters=None, top=1000, view_path=None) -> TimeRegistrationList:
        '''
            Returns a Time Registration list from a given sharepoint_site using its guid

            Returns a subset of items from a list

            sharepoint_site: The sharepoint_site containing the list
            sp_list: the guid of the list to retrieve items from
            filters: query filters
            top: Maximum items to query from the list
        '''

        return self.get_list(sharepoint_site, sp_list, filters, top, view_path, SPListType=TimeRegistrationList)

    def get_time_registration_list_by_name(self, sharepoint_site, sp_list_name: str, filters=None, top=1000, view_path=None) -> TimeRegistrationList:
        '''
            Returns a Time Registration list from a given sharepoint_site filtering by list name

            sharepoint_site: The sharepoint_site containing the list
            sp_list_name: the name of the list
            filters: query filters
            top: Maximum items to query from the list
        '''
        return self.get_list_by_name(sharepoint_site, sp_list_name, filters, top, view_path, SPListType=TimeRegistrationList)

    # SP Files

    def folder_exists(self, sharepoint_site, folder, in_doc_lib=True):
        '''
            Returns a boolean to indicate whether a folder exists
        '''

        if in_doc_lib:
            folder = os.path.join("DocumentLibrary", folder)

        r = self._api_get_call(f"{self.sharepoint_url}/cases/{sharepoint_site}/_api/Web/GetFolderByServerRelativeUrl('/cases/{sharepoint_site}/{folder}')/ListItemAllFields")

        return False if ("ListItemAllFields" in r.json()["d"] and r.json()["d"]["ListItemAllFields"] is None) else True

    def create_new_folder(self, sharepoint_site, folder, new_folder, in_doc_lib=True):
        '''
            Creates a new folder
        '''

        if in_doc_lib:
            folder = os.path.join("DocumentLibrary", folder)

        r = self._api_post_call(f'{self.sharepoint_url}/cases/{sharepoint_site}/_api/contextinfo', {})
        form_digest_value = r.json()["d"]["GetContextWebInformation"]["FormDigestValue"]

        r = self._api_post_call(f"{self.sharepoint_url}/cases/{sharepoint_site}/_api/web/folders",
                form_digest_value=form_digest_value,
                post_data={
                    "ServerRelativeUrl": f'/cases/{sharepoint_site}/{folder}/{new_folder}',
                }
            )

        return r.status_code

    def get_files(self, sharepoint_site, folder, in_doc_lib=True):
        '''
            Returns a list of files from a folder in a given sharepoint_site
        '''

        if in_doc_lib:
            folder = os.path.join("DocumentLibrary", folder)

        r = self._api_get_call(f"{self.sharepoint_url}/cases/{sharepoint_site}/_api/Web/GetFolderByServerRelativeUrl('/cases/{sharepoint_site}/{folder}')/Files")

        return r.json()["d"]["results"]

    def get_file_content(self, sharepoint_site, folder, file, in_doc_lib=True):
        '''
            Downloads and saves a file from a folder in a given sharepoint_site
        '''

        if in_doc_lib:
            folder = os.path.join("DocumentLibrary", folder)

        r = self._api_get_call(f"{self.sharepoint_url}/cases/{sharepoint_site}/_api/Web/GetFolderByServerRelativeUrl('/cases/{sharepoint_site}/{folder}')/Files('{file}')/$value")
        return r._content


    def download_file(self, sharepoint_site, folder, file, out_file=None, in_doc_lib=True):
        '''
            Downloads and saves a file from a folder in a given sharepoint_site
        '''

        if in_doc_lib:
            folder = os.path.join("DocumentLibrary", folder)

        if not out_file:
            out_file = file

        file_content = self.get_file_content(sharepoint_site, folder, file, in_doc_lib)

        with open(out_file, 'wb') as f:
            f.write(file_content)

    def upload_file_content(self, sharepoint_site, folder, file, file_content, overwrite=False, in_doc_lib=True):
        '''
            Uploads a file to a folder in a given sharepoint_site
        '''

        if in_doc_lib:
            folder = os.path.join("DocumentLibrary", folder)

        r = self._api_post_call(f'{self.sharepoint_url}/cases/{sharepoint_site}/_api/contextinfo', {})
        form_digest_value = r.json()["d"]["GetContextWebInformation"]["FormDigestValue"]

        r = self._api_attachment_call(
            f"{self.sharepoint_url}/cases/{sharepoint_site}/_api/Web/GetFolderByServerRelativeUrl('/cases/{sharepoint_site}/{folder}')/Files/add(url='{file}',overwrite={str(overwrite).lower()})",
            file_content,
            form_digest_value)
        return r.json()

    def upload_file(self, sharepoint_site, folder, file, in_file=None, overwrite=False, in_doc_lib=True):
        '''
            Uploads a file to a folder in a given sharepoint_site
        '''

        if in_doc_lib:
            folder = os.path.join("DocumentLibrary", folder)

        if not in_file:
            in_file = file
        with open(in_file, 'rb') as f:
            file_content = f.read()

        return self.upload_file_content(sharepoint_site, folder, file, file_content, overwrite, False)

    def delete_file(self, sharepoint_site, folder, file, in_doc_lib=True):
        '''
            Uploads a file to a folder in a given sharepoint_site
        '''

        if in_doc_lib:
            folder = os.path.join("DocumentLibrary", folder)

        r = self._api_post_call(f'{self.sharepoint_url}/cases/{sharepoint_site}/_api/contextinfo', {})
        form_digest_value = r.json()["d"]["GetContextWebInformation"]["FormDigestValue"]

        r = self._api_attachment_call(
            url=f"{self.sharepoint_url}/cases/{sharepoint_site}/_api/Web/GetFolderByServerRelativeUrl('/cases/{sharepoint_site}/{folder}')/Files('{file}')",
            post_data=None,
            form_digest_value=form_digest_value,
            x_http_method='DELETE')

    def copy_file(self, sharepoint_site, folder, file, out_folder=None, out_file=None, overwrite=False, in_doc_lib=True):
        '''
            Downloads and saves a file from a folder in a given sharepoint_site
        '''

        if in_doc_lib:
            folder = os.path.join("DocumentLibrary", folder)

        if not out_file:
            out_file = 'copy of '+file

        if not out_folder:
            out_folder = folder

        r = self._api_post_call(f'{self.sharepoint_url}/cases/{sharepoint_site}/_api/contextinfo', {})
        form_digest_value = r.json()["d"]["GetContextWebInformation"]["FormDigestValue"]

        in_path = f"/cases/{sharepoint_site}/{folder}"
        out_path = f"/cases/{sharepoint_site}/{out_folder}/{out_file}"

        r = self._api_attachment_call(
            f"{self.sharepoint_url}/cases/{sharepoint_site}/_api/Web/GetFolderByServerRelativeUrl('{in_path}')/Files('{file}')/copyto(strnewurl='{out_path}',boverwrite={str(overwrite).lower()})",
             None,
             form_digest_value)

    # STATIC METHODS

    @staticmethod
    def py2sp_conditional(conditional: str):
        ''''''
        return conditional.replace('==', 'eq').replace('!=', 'ne').replace('>=', 'ge').replace('<=', 'le').replace('>', 'gt').replace('<', 'lt')



from setuptools import setup

__version__ = "0.0.10"

with open("README.md", "r") as readme_file:
    long_description = readme_file.read()

packages = ["sharepoint_api"]

setup(
    name="sharepoint_v1_api",
    version=__version__,
    description="",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Aske Bluhme Klok",
    license="MIT",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    keywords="Sharepoint",
    packages=packages,
    install_requires=["requests", "requests-ntlm"],
    python_requires=">=3.6.8"
)